/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Sept. 2005

 File          : UART1.c

 Hardware      : Applicable to ADuC702x rev H or I silicon
                 Currently targetting ADuC7026.

 Description   : This code demonstrates basic UART functionality.
				 The baudrate is calculated with the following formula:
 
					DL = HCLK										   
						_______										   
						Baudrate * 2 *16							   
                                          																   
***********************************************************************/

#include <ADuC7020.h>
void wait(unsigned int);
void loop(void); 	
extern int write (int file, char * ptr, int len);	// Functions used to 
extern int getchar (void);							// to output data 	 
extern int putchar(int);                     		// Write character to Serial Port  
extern int printf(const char *format, ...);
extern void Serial_Setup(void);
void Interrupt_Setup(void);	
void Timer_Setup(void);	
void IRQ_Handler(void) __irq;
void FIQ_Handler(void) __fiq;       

/* interrupt stuff */
#define PRINT 0x01 /* background printing */
#define RTF 0x02	/* real time functions */
long systime;   /* timer ticks */
char systasks;

int main (void)  {
  unsigned char jchar = 0x30;

	char output1[13] = "Hello World\n";
	int counter = 0;
	GP4DAT = 0x04000000;			// P4.2 configured as an output. LED is turned on
  Serial_Setup();  /* initialize serial port */
  Timer_Setup();
  Interrupt_Setup();	
	printf("About to start RealTime() and background printing:\n");
	systasks = PRINT | RTF;   /* enable printing and real time functions */
 wait(200);   /* let sign on message print */
 printf("Hello, ee192 sp2010 !\n\r");
  loop();
	
	
	
	while(1)
	{

		GP4DAT ^= 0x00040000;		// Complement P4.2
		write(0,output1,13);		// Output Data
		jchar = getchar();			// RX Data, Single Byte
		write(0,&jchar,1);			// Output Rxed Data
		printf("hello, testing 123, counter = %u\n",counter);
		counter++;
	}
}

void loop()
{ int i;

/* these variables are updated by the real-time clock at 1 ms interval */

	printf("systime =0x%x\n", systime);
        printf("numbers: \n\r");
        for(i=0; i<20; i++)
	  { printf(" %x ", i); 
	    wait(200); }
        printf("\n\r");
	wait(2000);
	printf("systime=0x%x \n\r",
			systime);
	wait(2000);	/* wait for print buffer to empty before exiting */

}
/* kill some time waiting for real time ticks, wait in milliseconds */
/* note that systime is 32 bit value - it is handled by multiple 16 bit instructions - so make atomic */
#pragma ot(0)   /* disable optimization as it eliminates systime reading */

void wait(unsigned int n)
{ long start, end;
  long time;  /* copy of systime */
/* handle times greater than 65 seconds */
  if(n > 32767)
  { printf("Too long to wait!\n"); return; }

IRQCLR = RTOS_TIMER_BIT; /* mask RT interrupt so systime read is atomic */
 start = systime;
IRQEN |= RTOS_TIMER_BIT;  /* re-enable RT interrupt */
 end = start + n;
  if(start + n < start)    /* over flow */
  { printf("System time overflow!!!\n"); }
  else
  { 
IRQCLR = RTOS_TIMER_BIT; /* mask RT interrupt so systime read is atomic */
    time = systime;
 IRQEN |= RTOS_TIMER_BIT;  /* re-enable RT interrupt */
	while(end > time)	/* make sure not optimized away */
	{ 
      time = systime;	  /* assume can be read atomically */
	 
	}
  }
  printf("wait: start %lx end %lx systime %lx\n",
       start, end, systime);

}

void Interrupt_Setup(void){
//	IRQ = My_IRQ_Function;					// Specify Interrupt Service Rountine
//	FIQ = My_FIQ_Function;					// Specify Fast Interrupt Routine
	IRQEN = RTOS_TIMER_BIT;	// Specify which interrupts are enabled
											// (General Purpose Timer and Real Time Timer in this case)
											// Interrupt bit assignments are found in "irq.h"
//	FIQEN = ADC_BIT;						// Enable ADC FIQ
	printf("Interrupt Setup Complete\n");
	return;
}

void Timer_Setup(void){
 	T0LD = 20000;						// Set 16bit counter value
	T0CON = 0xC0;					    // Timer 0 enabled with no prescalar
	printf("Timer Setup Complete\n");

}


/*********************************************************************
 INTERRUPT SERVICE ROUTINE

 Description: The function is executed everytime an interrupt occurs.
 			  When an interrupt occurs, IRQSTA will be ON as well as
			  bit corresponding to the device that generated the interrupt.
			  For example, when the General Purpose Timer generates an
			  interrupt request, IRQSTA & GP_TIMER_BIT will both be ON.
			  Thus the follow condition will be true:
			  if((IRQSTA & GP_TIMER_BIT)!=0)
			  One can then service the General Purpose Timer interrupt
			  from within that 'if' statement.
*********************************************************************/



void IRQ_Handler(void) __irq {
	if((IRQSTA & RTOS_TIMER_BIT)!=0){
	/* place code to service the Real Time OS Timer interrupt here */
	 systime++;
	 T0CLRI = 0;	   //clear interrupt  for next time 
	return;
	}
	// catch other IRQ here
	return;
}

/*********************************************************************
 FAST INTERRUPT SERVICE ROUTINE

 Description: The function is executed everytime an fast interrupt request
 			  is generated. When a fast interrupt occurs, FIQSTA will be ON
			  as well as the bit corresponding to the device that generated
			  the interrupt. In this example, the ADC generates a fast interrupt
			  request. The interrupt handler takes the ADC data and outputs it
			  to the DAC, a red LED on the MiniKit.
*********************************************************************/

void My_FIQ_Function() __fiq {						// example with one FIQ			
	if ((FIQSTA & ADC_BIT) != 0){ 				// ADC FIQ
	
	}
	return;
}


